@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/kauth/kaprocs.c,v 1.16 2004/04/20 14:56:16 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/kauth/kaprocs.c,v 1.17 2004/08/19 01:07:47 kolya Exp $");
 
 #include <afs/stds.h>
 #include <errno.h>
@@ -1032,7 +1032,8 @@ PrepareTicketAnswer(oanswer, challenge, ticket, ticketLen, sessionKey, start,
 
     code = KAANSWERTOOLONG;
     if (oanswer->MaxSeqLen <
-	sizeof(struct ka_ticketAnswer) - 5 * MAXKTCNAMELEN - MAXKTCTICKETLEN + ticketLen)
+	sizeof(struct ka_ticketAnswer) - 5 * MAXKTCNAMELEN - MAXKTCTICKETLEN +
+	ticketLen)
 	return code;
 
     answer = (struct ka_ticketAnswer *)oanswer->SeqBody;
@@ -1958,7 +1959,8 @@ GetTicket(version, call, kvno, authDomain, aticket, sname, sinstance, atimes,
     case 0:
 	code = KAANSWERTOOLONG;
 	if (oanswer->MaxSeqLen <
-	    sizeof(struct ka_getTicketAnswer) - 5 * MAXKTCNAMELEN - MAXKTCTICKETLEN + ticketLen)
+	    sizeof(struct ka_getTicketAnswer) - 5 * MAXKTCNAMELEN -
+	    MAXKTCTICKETLEN + ticketLen)
 	    goto abort;
 
 	answer = (struct ka_getTicketAnswer *)oanswer->SeqBody;
