@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/FBSD/rx_knet.c,v 1.13 2003/07/15 23:16:18 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/FBSD/rx_knet.c,v 1.15 2005/04/14 02:25:16 shadow Exp $");
 
 #ifdef AFS_FBSD40_ENV
 #include <sys/malloc.h>
@@ -83,10 +83,20 @@ osi_StopListener(void)
 {
     struct proc *p;
 
+    /*
+     * Have to drop global lock to safely do this.
+     * soclose() is currently protected by Giant,
+     * but pfind and psignal are MPSAFE.
+     */
+    AFS_GUNLOCK();
     soclose(rx_socket);
     p = pfind(rxk_ListenerPid);
     if (p)
 	psignal(p, SIGUSR1);
+#ifdef AFS_FBSD50_ENV
+    PROC_UNLOCK(p);
+#endif
+    AFS_GLOCK();
 }
 
 int
@@ -273,7 +283,6 @@ rxk_input(struct mbuf *am, int iphlen)
 		 */
 		data_len = ntohs(tu->uh_ulen);
 		data_len -= 8;
-		AFS_RXGLOCK();
 		if (!(*rxk_GetPacketProc) (&phandle, data_len)) {
 		    if (rx_mb_to_packet(am, m_freem, 28, data_len, phandle)) {
 			/* XXX should just increment counter here.. */
@@ -284,7 +293,6 @@ rxk_input(struct mbuf *am, int iphlen)
 						  rxk_portRocks[i], data_len);
 		} else
 		    m_freem(am);
-		AFS_RXGUNLOCK();
 		USERPRI;
 		return;
 	    }
