@@ -13,7 +13,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/host.c,v 1.93 2006/04/04 20:51:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/host.c,v 1.93.2.5 2006/06/07 04:56:29 shadow Exp $");
 
 #include <stdio.h>
 #include <errno.h>
@@ -175,6 +175,8 @@ GetCE()
 static void
 FreeCE(register struct client *entry)
 {
+    entry->VenusEpoch = 0;
+    entry->sid = 0;
     entry->next = CEFree;
     CEFree = entry;
     CEs--;
@@ -749,10 +751,6 @@ h_TossStuff_r(register struct host *host)
 	    if ((client->ViceId != ANONYMOUSID) && client->CPS.prlist_val)
 		free(client->CPS.prlist_val);
 	    client->CPS.prlist_val = NULL;
-	    if (client->tcon) {
-		rx_SetSpecific(client->tcon, rxcon_client_key, (void *)0);
-		rx_PutConnection(client->tcon);
-	    }
 	    CurrentConnections--;
 	    *cp = client->next;
 	    ReleaseWriteLock(&client->lock);
@@ -775,18 +773,6 @@ h_TossStuff_r(register struct host *host)
 	    Console--;
 	if ((rxconn = host->callback_rxcon)) {
 	    host->callback_rxcon = (struct rx_connection *)0;
-	    /*
-	     * If rx_DestroyConnection calls h_FreeConnection we will
-	     * deadlock on the host_glock_mutex. Work around the problem
-	     * by unhooking the client from the connection before
-	     * destroying the connection.
-	     */
-	    client = rx_GetSpecific(rxconn, rxcon_client_key);
-	    if (client && client->tcon == rxconn) {
-		rx_PutConnection(client->tcon);
-		client->tcon = NULL;
-	    }
-	    rx_SetSpecific(rxconn, rxcon_client_key, (void *)0);
 	    rx_DestroyConnection(rxconn);
 	}
 	if (host->hcps.prlist_val)
@@ -844,26 +830,6 @@ h_TossStuff_r(register struct host *host)
     }
 }				/*h_TossStuff_r */
 
-
-/* Called by rx when a server connection disappears */
-int
-h_FreeConnection(struct rx_connection *tcon)
-{
-    register struct client *client;
-
-    client = (struct client *)rx_GetSpecific(tcon, rxcon_client_key);
-    if (client) {
-	H_LOCK;
-	if (client->tcon == tcon) {
-	    rx_PutConnection(client->tcon);
-	    client->tcon = NULL;
-	}
-	H_UNLOCK;
-    }
-    return 0;
-}				/*h_FreeConnection */
-
-
 /* h_Enumerate: Calls (*proc)(host, held, param) for at least each host in the
  * system at the start of the enumeration (perhaps more).  Hosts may be deleted
  * (have delete flag set); ditto for clients.  (*proc) is always called with
@@ -978,6 +944,12 @@ h_AddHostToHashTable_r(afs_uint32 addr, afs_uint16 port, struct host *host)
     /* hash into proper bucket */
     index = h_HashIndex(addr);
 
+    /* don't add the same entry multiple times */
+    for (chain = hostHashTable[index]; chain; chain = chain->next) {
+	if (chain->hostPtr == host && chain->addr == addr && chain->port == port)
+	    return;
+    }
+
     /* insert into beginning of list for this bucket */
     chain = (struct h_hashChain *)malloc(sizeof(struct h_hashChain));
     if (!chain) {
@@ -989,7 +961,6 @@ h_AddHostToHashTable_r(afs_uint32 addr, afs_uint16 port, struct host *host)
     chain->addr = addr;
     chain->port = port;
     hostHashTable[index] = chain;
-
 }
 
 /*
@@ -1346,52 +1317,54 @@ h_GetHost_r(struct rx_connection *tcon)
 		    h_Lock_r(oldHost);
 
                     if (oldHost->interface) {
+			int code2;
 			afsUUID uuid = oldHost->interface->uuid;
                         cb_conn = oldHost->callback_rxcon;
                         rx_GetConnection(cb_conn);
 			rx_SetConnDeadTime(cb_conn, 2);
 			rx_SetConnHardDeadTime(cb_conn, AFS_HARDDEADTIME);
 			H_UNLOCK;
-			code = RXAFSCB_ProbeUuid(cb_conn, &uuid);
+			code2 = RXAFSCB_ProbeUuid(cb_conn, &uuid);
 			H_LOCK;
 			rx_SetConnDeadTime(cb_conn, 50);
 			rx_SetConnHardDeadTime(cb_conn, AFS_HARDDEADTIME);
                         rx_PutConnection(cb_conn);
                         cb_conn=NULL;
-			if (code && MultiProbeAlternateAddress_r(oldHost)) {
+			if (code2) {
+			    /* The primary address is either not responding or
+			     * is not the client we are looking for.  
+			     * MultiProbeAlternateAddress_r() will remove the
+			     * alternate interfaces that do not have the same
+			     * Uuid. */
+			    ViceLog(0,("CB: ProbeUuid for %s:%d failed %d\n",
+					 afs_inet_ntoa_r(oldHost->host, hoststr),
+					 ntohs(oldHost->port),code2));
+			    MultiProbeAlternateAddress_r(oldHost);
                             probefail = 1;
                         }
                     } else {
                         probefail = 1;
                     }
 
-                    if (probefail) {
-                        /* The old host is either does not have a Uuid,
-                         * is not responding to Probes, 
-                         * or does not have a matching Uuid. 
-                         * Delete it! */
-                        oldHost->hostFlags |= HOSTDELETED;
-                        h_Unlock_r(oldHost);
-			/* Let the holder be last release */
-			if (!oheld) {
-			    h_Release_r(oldHost);
-			}
-			oldHost = NULL;
-                    }
-                }
-		if (oldHost) {
 		    /* This is a new address for an existing host. Update
 		     * the list of interfaces for the existing host and
 		     * delete the host structure we just allocated. */
 		    if (oldHost->host != haddr || oldHost->port != hport) {
+			struct rx_connection *rxconn;
+
 			ViceLog(25,
 				("CB: new addr %s:%d for old host %s:%d\n",
 				  afs_inet_ntoa_r(haddr, hoststr),
 				  ntohs(hport), 
 				  afs_inet_ntoa_r(oldHost->host, hoststr2),
 				  ntohs(oldHost->port)));
-			if (oldHost->host == haddr) {
-			    /* We have just been contacted by a client behind a NAT */
+			if (probefail || oldHost->host == haddr) {
+			    /* The probe failed which means that the old address is 
+			     * either unreachable or is not the same host we were just
+			     * contacted by.  We will also remove addresses if only
+			     * the port has changed because that indicates the client
+			     * is behind a NAT. 
+			     */
 			    removeInterfaceAddr_r(oldHost, oldHost->host, oldHost->port);
 			} else {
 			    int i, found;
@@ -1414,6 +1387,22 @@ h_GetHost_r(struct rx_connection *tcon)
 			addInterfaceAddr_r(oldHost, haddr, hport);
 			oldHost->host = haddr;
 			oldHost->port = hport;
+			rxconn = oldHost->callback_rxcon;
+			oldHost->callback_rxcon = host->callback_rxcon;
+			host->callback_rxcon = NULL;
+			
+			if (rxconn) {
+			    struct client *client;
+			    /*
+			     * If rx_DestroyConnection calls h_FreeConnection we will
+			     * deadlock on the host_glock_mutex. Work around the problem
+			     * by unhooking the client from the connection before
+			     * destroying the connection.
+			     */
+			    client = rx_GetSpecific(rxconn, rxcon_client_key);
+			    rx_SetSpecific(rxconn, rxcon_client_key, (void *)0);
+			    rx_DestroyConnection(rxconn);
+			}
 		    }
 		    host->hostFlags |= HOSTDELETED;
 		    h_Unlock_r(host);
@@ -1640,7 +1629,8 @@ h_FindClient_r(struct rx_connection *tcon)
     int created = 0;
 
     client = (struct client *)rx_GetSpecific(tcon, rxcon_client_key);
-    if (client) {
+    if (client && client->sid == rxr_CidOf(tcon) 
+	&& client->VenusEpoch == rxr_GetEpoch(tcon)) {
 	client->refCount++;
 	h_Hold_r(client->host);
 	if (!client->deleted && client->prfail != 2) {	
@@ -1713,26 +1703,6 @@ h_FindClient_r(struct rx_connection *tcon)
 	for (client = host->FirstClient; client; client = client->next) {
 	    if (!client->deleted && (client->sid == rxr_CidOf(tcon))
 		&& (client->VenusEpoch == rxr_GetEpoch(tcon))) {
-		if (client->tcon && (client->tcon != tcon)) {
-		    ViceLog(0,
-			    ("*** Vid=%d, sid=%x, tcon=%x, Tcon=%x ***\n",
-			     client->ViceId, client->sid, client->tcon,
-			     tcon));
-		    oldClient =
-			(struct client *)rx_GetSpecific(client->tcon,
-							rxcon_client_key);
-		    if (oldClient) {
-			if (oldClient == client) {
-			    rx_SetSpecific(client->tcon, rxcon_client_key,
-					   NULL);
-			} else
-			    ViceLog(0,
-				    ("Client-conn mismatch: CL1=%x, CN=%x, CL2=%x\n",
-				     client, client->tcon, oldClient));
-		    }
-		    rx_PutConnection(client->tcon);
-		    client->tcon = (struct rx_connection *)0;
-		}
 		client->refCount++;
 		H_UNLOCK;
 		ObtainWriteLock(&client->lock);
@@ -1825,7 +1795,8 @@ h_FindClient_r(struct rx_connection *tcon)
      * the RPC from the other client structure's rock.
      */
     oldClient = (struct client *)rx_GetSpecific(tcon, rxcon_client_key);
-    if (oldClient && oldClient->tcon == tcon) {
+    if (oldClient && oldClient->sid == rxr_CidOf(tcon)
+	&& oldClient->VenusEpoch == rxr_GetEpoch(tcon)) {
 	char hoststr[16];
 	if (!oldClient->deleted) {
 	    /* if we didn't create it, it's not ours to put back */
@@ -1839,9 +1810,6 @@ h_FindClient_r(struct rx_connection *tcon)
 		    free(client->CPS.prlist_val);
 		client->CPS.prlist_val = NULL;
 		client->CPS.prlist_len = 0;
-		if (client->tcon) {
-		    rx_SetSpecific(client->tcon, rxcon_client_key, (void *)0);
-		}
 	    }
 	    /* We should perhaps check for 0 here */
 	    client->refCount--;
@@ -1854,8 +1822,6 @@ h_FindClient_r(struct rx_connection *tcon)
 	    oldClient->refCount++;
 	    client = oldClient;
 	} else {
-	    rx_PutConnection(oldClient->tcon);
-	    oldClient->tcon = (struct rx_connection *)0;
 	    ViceLog(0, ("FindClient: deleted client %x(%x) already had conn %x (host %s:%d), stolen by client %x(%x)\n", 
 			oldClient, oldClient->sid, tcon, 
 			afs_inet_ntoa_r(rxr_HostOf(tcon), hoststr),
@@ -1872,8 +1838,6 @@ h_FindClient_r(struct rx_connection *tcon)
 	h_Unlock_r(host);
 	CurrentConnections++;	/* increment number of connections */
     }
-    rx_GetConnection(tcon);
-    client->tcon = tcon;
     rx_SetSpecific(tcon, rxcon_client_key, client);
     ReleaseWriteLock(&client->lock);
 
@@ -1905,30 +1869,20 @@ GetClient(struct rx_connection *tcon, struct client **cp)
     H_LOCK;
     *cp = NULL;
     client = (struct client *)rx_GetSpecific(tcon, rxcon_client_key);
-    if (client == NULL || client->tcon == NULL) {
+    if (client == NULL) {
 	ViceLog(0,
 		("GetClient: no client in conn %x (host %x:%d), VBUSYING\n",
 		 tcon, rxr_HostOf(tcon),ntohs(rxr_PortOf(tcon))));
 	H_UNLOCK;
 	return VBUSY;
     }
-    if (rxr_CidOf(client->tcon) != client->sid) {
+    if (rxr_CidOf(tcon) != client->sid || rxr_GetEpoch(tcon) != client->VenusEpoch) {
 	ViceLog(0,
 		("GetClient: tcon %x tcon sid %d client sid %d\n",
-		 client->tcon, rxr_CidOf(client->tcon), client->sid));
+		 tcon, rxr_CidOf(tcon), client->sid));
 	H_UNLOCK;
 	return VBUSY;
     }
-    if (!(client && client->tcon && rxr_CidOf(client->tcon) == client->sid)) {
-	if (!client)
-	    ViceLog(0, ("GetClient: no client in conn %x\n", tcon));
-	else
-	    ViceLog(0,
-		    ("GetClient: tcon %x tcon sid %d client sid %d\n",
-		     client->tcon, client->tcon ? rxr_CidOf(client->tcon)
-		     : -1, client->sid));
-	assert(0);
-    }
     if (client && client->LastCall > client->expTime && client->expTime) {
 	char hoststr[16];
 	ViceLog(1,
@@ -2024,7 +1978,6 @@ h_PrintClient(register struct host *host, int held, StreamHandle_t * file)
     (void)STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
     for (client = host->FirstClient; client; client = client->next) {
 	if (!client->deleted) {
-	    if (client->tcon) {
 		expTime = client->expTime;
 		(void)afs_snprintf(tmpStr, sizeof tmpStr,
 				   "    user id=%d,  name=%s, sl=%s till %s",
@@ -2037,12 +1990,6 @@ h_PrintClient(register struct host *host, int held, StreamHandle_t * file)
 							 sizeof(tbuffer))
 				   : "No Limit\n");
 		(void)STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
-	    } else {
-		(void)afs_snprintf(tmpStr, sizeof tmpStr,
-				   "    user=%s, no current server connection\n",
-				   h_UserName(client));
-		(void)STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
-	    }
 	    (void)afs_snprintf(tmpStr, sizeof tmpStr, "      CPS-%d is [",
 			       client->CPS.prlist_len);
 	    (void)STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
@@ -2980,6 +2927,8 @@ CheckHost(register struct host *host, int held)
 		}
 	    } else {
 		if (!(host->hostFlags & VENUSDOWN) && host->cblist) {
+		    char hoststr[16];
+		    (void)afs_inet_ntoa_r(host->host, hoststr);
 		    if (host->interface) {
 			afsUUID uuid = host->interface->uuid;
 			H_UNLOCK;
@@ -2987,11 +2936,8 @@ CheckHost(register struct host *host, int held)
 			H_LOCK;
 			if (code) {
 			    if (MultiProbeAlternateAddress_r(host)) {
-				char hoststr[16];
-				(void)afs_inet_ntoa_r(host->host, hoststr);
-				ViceLog(0,
-					("ProbeUuid failed for host %s:%d\n",
-					 hoststr, ntohs(host->port)));
+				ViceLog(0,("CheckHost: Probing all interfaces of host %s:%d failed, code %d\n",
+					    hoststr, ntohs(host->port), code));
 				host->hostFlags |= VENUSDOWN;
 			    }
 			}
@@ -3000,11 +2946,9 @@ CheckHost(register struct host *host, int held)
 			code = RXAFSCB_Probe(cb_conn);
 			H_LOCK;
 			if (code) {
-			    char hoststr[16];
-			    (void)afs_inet_ntoa_r(host->host, hoststr);
 			    ViceLog(0,
-				    ("Probe failed for host %s:%d\n", hoststr,
-				     ntohs(host->port)));
+				    ("CheckHost: Probe failed for host %s:%d, code %d\n", 
+				     hoststr, ntohs(host->port), code));
 			    host->hostFlags |= VENUSDOWN;
 			}
 		    }
@@ -3065,22 +3009,23 @@ initInterfaceAddr_r(struct host *host, struct interfaceAddr *interf)
     afs_uint16 myPort;
     int found;
     struct Interface *interface;
+    char hoststr[16];
 
     assert(host);
     assert(interf);
 
-    ViceLog(125,
-	    ("initInterfaceAddr : host %x numAddr %d\n", host->host,
-	     interf->numberOfInterfaces));
-
     number = interf->numberOfInterfaces;
     myAddr = host->host;	/* current interface address */
     myPort = host->port;	/* current port */
 
+    ViceLog(125,
+	    ("initInterfaceAddr : host %s:%d numAddr %d\n", 
+	      afs_inet_ntoa_r(myAddr, hoststr), ntohs(myPort), number));
+
     /* validation checks */
     if (number < 0 || number > AFS_MAX_INTERFACE_ADDR) {
 	ViceLog(0,
-		("Number of alternate addresses returned is %d\n", number));
+		("Invalid number of alternate addresses is %d\n", number));
 	return -1;
     }
 
@@ -3136,7 +3081,6 @@ initInterfaceAddr_r(struct host *host, struct interfaceAddr *interf)
     host->interface = interface;
 
     for (i = 0; i < host->interface->numberOfInterfaces; i++) {
-	char hoststr[16];
 	ViceLog(125, ("--- alt address %s:%d\n", 
 		       afs_inet_ntoa_r(host->interface->interface[i].addr, hoststr),
 		       ntohs(host->interface->interface[i].port)));
